#!/usr/bin/env python
# -*- coding: utf-8 -*-

from time import time
import logging

_logger = logging.getLogger(__name__)
ENABLE_DEBUG_LOG = False


class RateLimiter(object):
    """!
    Gradienten-Begrenzer für die Temperatur.
    """
    def __init__(self, rate, start_value):
        """!
        Initialisiert den Gradienten-Begrenzer.
        Benötigt gewünschten Gradienten und Startwert.
        @param rate: Gradient in K\s
        @param start_value: Startwert der Rampe
        """
        self._rate = rate
        self._output = start_value
        self._last_time = time()

    def rate(self, target_value):
        """!
        Berechnet den nächsten Sollwert auf Basis des gewünschten Gradienten.
        @param target_value: Ziel-Sollwert
        @return Neuer Sollwert
        """
        _step = self._rate * (time() - self._last_time)
        self._last_time = time()

        if target_value > self._output:
            self._output += _step
            self._output = min(target_value, self._output)
        elif target_value < self._output:
            self._output -= _step
            self._output = max(target_value, self._output)
        if ENABLE_DEBUG_LOG:
            _logger.debug("time: {}, step {}".format(self._last_time, _step))

        return round(self._output, 1)
